/**
  *  \file t_ptrvector.cc
  */

#include "cpluslib/ptrvector.h"

#include "tests.h"

namespace {
    class Obj {
        Obj(const Obj&);
        Obj& operator=(const Obj&);
     public:
        int n;
        static int live;
        Obj(int n)
            : n(n) { ++live; }
        ~Obj()
            { --live; }
    };
    int Obj::live = 0;
}

void
TestPtrVector::test1()
{
    // Blank vector
    ptr_vector<Obj> pv;
    TS_ASSERT(pv.empty());
    TS_ASSERT_EQUALS(pv.size(), 0);
    TS_ASSERT_EQUALS(Obj::live, 0);

    // Add three objects
    pv.push_back(new Obj(1));
    pv.push_back(new Obj(2));
    pv.push_back(new Obj(3));   // 1 2 3
    TS_ASSERT(!pv.empty());
    TS_ASSERT_EQUALS(pv.size(), 3);
    TS_ASSERT_EQUALS(Obj::live, 3);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1]->n, 2);
    TS_ASSERT_EQUALS(pv[2]->n, 3);
    TS_ASSERT_EQUALS(pv[0], pv.front());
    TS_ASSERT_EQUALS(pv[2], pv.back());

    // Remove one
    pv.pop_back();              // 1 2
    TS_ASSERT_EQUALS(pv.size(), 2);
    TS_ASSERT_EQUALS(Obj::live, 2);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1]->n, 2);

    // Add some more
    pv.push_back(new Obj(4));
    pv.push_back(new Obj(5));
    pv.push_back(new Obj(6));
    pv.push_back(new Obj(7));   // 1 2 4 5 6 7
    TS_ASSERT_EQUALS(Obj::live, 6);
    TS_ASSERT_EQUALS(pv.front()->n, 1);
    TS_ASSERT_EQUALS(pv.back()->n,  7);

    // Swap some
    pv.swap_elems(1, 4);        // 1 6 4 5 2 7
    TS_ASSERT_EQUALS(Obj::live, 6);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1]->n, 6);
    TS_ASSERT_EQUALS(pv[2]->n, 4);
    TS_ASSERT_EQUALS(pv[3]->n, 5);
    TS_ASSERT_EQUALS(pv[4]->n, 2);
    TS_ASSERT_EQUALS(pv[5]->n, 7);

    // Delete one
    pv.erase(pv.begin() + 2);   // 1 6 5 2 7
    TS_ASSERT_EQUALS(Obj::live, 5);
    TS_ASSERT_EQUALS(pv.size(), 5);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1]->n, 6);
    TS_ASSERT_EQUALS(pv[2]->n, 5);
    TS_ASSERT_EQUALS(pv[3]->n, 2);
    TS_ASSERT_EQUALS(pv[4]->n, 7);

    // Delete two
    pv.erase(pv.begin() + 3, pv.end());  // 1 6 5
    TS_ASSERT_EQUALS(Obj::live, 3);
    TS_ASSERT_EQUALS(pv.size(), 3);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1]->n, 6);
    TS_ASSERT_EQUALS(pv[2]->n, 5);

    // Delete another one
    delete pv.extract_elem(1);
    TS_ASSERT_EQUALS(Obj::live, 2);      // 1 nil 5
    TS_ASSERT_EQUALS(pv.size(), 3);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1],    (Obj*)0);
    TS_ASSERT_EQUALS(pv[2]->n, 5);

    pv.erase(pv.begin()+1);              // 1 5
    TS_ASSERT_EQUALS(Obj::live, 2);
    TS_ASSERT_EQUALS(pv.size(), 2);
    TS_ASSERT_EQUALS(pv[0]->n, 1);
    TS_ASSERT_EQUALS(pv[1]->n, 5);
}
